"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hashDistroLeaf = exports.prepareDistribution = void 0;
const umi_1 = require("@metaplex-foundation/umi");
const merkleTree_1 = require("./merkleTree");
function prepareDistribution(recipients) {
    // Validate amounts
    const invalidAmountRecipient = recipients.find((recipient) => recipient.amount <= 0);
    if (invalidAmountRecipient) {
        throw new Error(`Amount must be greater than 0 for recipient ${invalidAmountRecipient.address}`);
    }
    const leafs = recipients.map(hashDistroLeaf);
    const tree = (0, merkleTree_1.createMerkleTree)(leafs);
    const merkleRoot = (0, merkleTree_1.getRoot)(tree);
    return {
        root: merkleRoot,
        proofs: leafs.map((_, i) => (0, merkleTree_1.getProof)(tree, i)),
    };
}
exports.prepareDistribution = prepareDistribution;
function hashDistroLeaf(recipient) {
    const pubkey = (0, umi_1.publicKey)(recipient.address);
    const pubkeyBytes = Buffer.from((0, umi_1.publicKeyBytes)(pubkey));
    // Convert amount to u64 little-endian bytes (8 bytes)
    const amountBytes = Buffer.alloc(8);
    amountBytes.writeBigUInt64LE(BigInt(recipient.amount));
    const nonceBytes = Buffer.alloc(8);
    nonceBytes.writeBigUInt64LE(BigInt(recipient.nonce ?? 0));
    return Buffer.concat([pubkeyBytes, amountBytes, nonceBytes]);
}
exports.hashDistroLeaf = hashDistroLeaf;
//# sourceMappingURL=distro.js.map