"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.verifyClaim = exports.verifyProof = exports.getHexProof = exports.getHexRoot = exports.getProof = exports.getRoot = exports.internalHash = exports.nodeHash = exports.createMerkleTree = void 0;
const sha3_1 = require("@noble/hashes/sha3");
const utils_1 = require("@noble/hashes/utils");
function createMerkleTree(leaves) {
    const tree = {
        leaves: leaves.slice(),
        layers: [],
    };
    let hashes = tree.leaves.map(nodeHash);
    while (hashes.length > 1) {
        tree.layers.push(hashes.slice());
        hashes = hashes.reduce((acc, cur, idx, arr) => {
            if (idx % 2 === 0) {
                const nxt = arr[idx + 1];
                acc.push(internalHash(cur, nxt));
            }
            return acc;
        }, Array());
    }
    // Push the final layer if there is a remaining hash.
    if (hashes.length === 1) {
        tree.layers.push(hashes.slice());
    }
    return tree;
}
exports.createMerkleTree = createMerkleTree;
function nodeHash(data) {
    const claimPrefix = new Uint8Array([0x63, 0x6c, 0x61, 0x69, 0x6d]); // ASCII bytes for "claim"
    const prefixed = (0, utils_1.concatBytes)(claimPrefix, new Uint8Array(data));
    return Buffer.from((0, sha3_1.keccak_256)(prefixed));
}
exports.nodeHash = nodeHash;
function internalHash(first, second) {
    if (!second)
        return first;
    const [fst, snd] = [first, second].sort(Buffer.compare);
    const prefixed = (0, utils_1.concatBytes)(new Uint8Array([0x01]), new Uint8Array(fst), new Uint8Array(snd));
    return Buffer.from((0, sha3_1.keccak_256)(prefixed));
}
exports.internalHash = internalHash;
function getRoot(tree) {
    return tree.layers[tree.layers.length - 1][0];
}
exports.getRoot = getRoot;
function getProof(tree, idx) {
    return tree.layers.reduce((proof, layer) => {
        // Calculate sibling index without using bitwise operators
        // const isEven = idx % 2 === 0;
        // const sibling = isEven ? idx + 1 : idx - 1;
        // eslint-disable-next-line no-bitwise
        const sibling = idx ^ 1;
        if (sibling < layer.length) {
            proof.push(layer[sibling]);
        }
        idx = Math.floor(idx / 2);
        return proof;
    }, []);
}
exports.getProof = getProof;
function getHexRoot(tree) {
    return getRoot(tree).toString('hex');
}
exports.getHexRoot = getHexRoot;
function getHexProof(tree, idx) {
    return getProof(tree, idx).map((el) => el.toString('hex'));
}
exports.getHexProof = getHexProof;
function verifyProof(tree, idx, proof, root) {
    const pair = nodeHash(tree.leaves[idx]);
    return proof
        .reduce((acc, item) => internalHash(acc, item), pair)
        .equals(root);
}
exports.verifyProof = verifyProof;
function verifyClaim(leaf, proof, root) {
    const pair = nodeHash(leaf);
    return proof
        .reduce((acc, item) => internalHash(acc, item), pair)
        .equals(root);
}
exports.verifyClaim = verifyClaim;
//# sourceMappingURL=merkleTree.js.map